#include <stdio.h>
#include<iostream>
#include <fstream>
#include<math.h>
#include<cmath>
#include<complex>

using namespace std;
typedef complex<long double> cld;
typedef long double ld;

#define ACCSMEAR    100   // for energy-integration accuracy to obtain functions smeared with a Gaussian
#define ACCTOT    1   // for angle-integration accuracy to obtain total cross section
#define CONST1 2000 // for integration routines - do not change
#define CONST2 10   // for integration routines - do not change

/*Basic function definitions*/
cld Power(cld a, cld b)
{return pow(a,b);}

cld Complex(ld a, ld b)
{return a+1i*b;}

cld Exp(cld a)
{cld x = a;
 return exp(x);}

cld Sqrt(cld a)
{cld x = a;
 return sqrt(x);}

/*Constants*/
cld E = Exp(1.l);
cld Pi = 3.14159265359l;
cld alpha = 1.l/137.l;
cld xMpro  = 0.9383l;// Proton mass
cld xMjpsi  = 3.097l;// J/psi mass
cld xFjpsi = 0.278l;// J/psi decay constant

/*---------------------------------------------------------
Main
---------------------------------------------------------*/
int main ()
{
	cld DXSFuncParam(cld * parameters, cld * input);//differential cross section
	cld DXSSmearFuncParam(cld * parameters, cld * input);//differential cross section with smearing
	cld TotXSFuncParam(cld * parameters, cld * input);//total cross section
	cld TotXSSmearFuncParam(cld * parameters, cld * input);//total cross section with smearing
/*=========================================================
=========================INPUT IN THE CODE=================
=========================================================*/
        cld eres = (pow(4.45l,2) - pow(xMpro,2))/(2.l*xMpro);//Example energy set to resonance energy
/*Next are the four-vectors:
- The first four components are the 4-vector of the photon in the lab frame
- The next four components are the 4-vector of the electron in the lab frame
- The last four components are the 4-vector of the positron in the lab frame
*/
        cld a[12] = {eres,0.l,0.l,eres,3.2037642820224725l,0.l,0.l,0.8201808183339612l,0.l,0.l,0.l,0.l};
/*Next are the input parameters:
- Pomeron parameter a_0 (unitless)
- Pomeron parameter a' (in GeV^-2)
- Pomeron parameter A_norm (unitless)
- Pomeron parameter s_t (in GeV^2)
- Pomeron parameter b_0 (in GeV^-2)
- Branching ratio (unitless)
- Mass of the P_c(4450) (in GeV)
- Width of the P_c(4450) (in GeV)
- Spin of the P_c(4450): has to be 3.l/2.l or 5.l/2.l
- Smearing due to experimental resolution (in GeV)
- Choice of the photocouplings: A12^2 + A32^2 is fixed by vector-meson dominance. Therefore choose A12/sqrt(A12^2+A32^2) between 0.l and 1.l.
*/
        cld b[11] = {1.115l,0.112l,0.156l,16.8l,1.01l,0.29l,4.45l,0.039l,3.l/2.l,0.l,0.71l};
/*=========================================================
=========================OUTPUT IN TERMINAL================
=========================================================*/
	cout << DXSFuncParam(a,b) << endl;
	cout << TotXSFuncParam(a,b) << endl;
/*=========================================================
=========================INPUT FILES=======================
=========================================================*/
	ifstream inputfile;
	inputfile.open("inputfile.txt");
	cld input[11];
	for(int i = 0; i < 11; i++){
	inputfile>>input[i];
	}
	inputfile.close();

	ifstream fourvecfile;
	fourvecfile.open("fourvecs.txt");
	cld fourvec[12];
	for(int i = 0; i < 12; i++){
	fourvecfile>>fourvec[i];
	}
	fourvecfile.close();
/*=========================================================
=========================OUTPUT FILE=======================
=========================================================*/
	ofstream outputfile;
	outputfile.open("outputfile.txt");
	outputfile << "Differential cross section: " << DXSFuncParam(fourvec,input) << endl;
	outputfile << "Total cross section: " << TotXSFuncParam(fourvec,input) << endl;
	outputfile.close();
//
	return 0;
}


/*=======================================================
==================Functions==============================
=======================================================*/

//Reads the four-vectors and the input parameters
//Returns the differential cross section
cld DXSFuncParam(cld * parameters, cld * input)
{
  cld DXSFunc(cld egam, cld cos, cld * input);
  cld DXSSmearFunc(cld egam, cld cos, cld * input);
  cld get_cosCM(cld * parameters);
  cld egam = parameters[0];
  cld cos = get_cosCM(parameters);
  cld result;
  if (input[9]==0.l){
	result = real(DXSFunc(egam,cos,input));}
  else{
	result = real(DXSSmearFunc(egam,cos,input));}
  return result;
}

//Reads the four-vectors and the input parameters
//Returns the total cross section
cld TotXSFuncParam(cld * parameters, cld * input)
{
  cld TotXSFunc(cld egam, cld * input);
  cld TotXSSmearFunc(cld egam, cld * input);
  cld egam = parameters[0];
  cld result;
  if (input[9]==0.l){
	result = real(TotXSFunc(egam,input));}
  else{
	result = real(TotXSSmearFunc(egam,input));}
  return result;
}	

/*=======================================================================================
Auxiliary functions
=======================================================================================*/

/*==============================================
  COMPLEX INTEGRATION
      1) DSG20R - preparing the array xd and np, i.e. the x-values where f(x) will be evaluated 
      2) DRG20_complex - summing the f(x) values */
void DSG20R(ld & xlow, ld & xhigh, ld & n, ld * xd, ld & np)
{
  ld y[CONST2] = { .9931285991, .9639719272, .9122344282, .8391169718,
		   .7463319064, .6360536807, .5108670019, .3737060887,
		   .2277858511, .0765265211};
  ld xn, dint, delt, orig, dorig;
  int i, i1, i2, j, j1, j2;
  np   = 20.l * n;
  xn   = n;
  dint = xhigh - xlow;
  dint = dint/xn;
  delt = dint/2.;
  orig = xlow - delt;
  i1   = -20;
  for(i = 0; i < int(n); i++){
    orig  = orig + dint;
    dorig = orig + orig;
    i1 = i1 + 20;
    i2 = i1 + 19;

    for(j = 0; j < CONST2; j++){
      j1 = i1 + j;
      j2 = i2 - j;
      xd[j1] = orig - delt * y[j];
      xd[j2] = dorig - xd[j1];
    }
  }
}

void DRG20_complex(ld & xlow, ld & xhigh, ld & n, cld * fd_c, cld & int_result)
{
  ld w[CONST2] = {.0176140071, .0406014298, .0626720483, .0832767415,
		  .1019301198, .1181945319, .1316886384, .1420961093,
		  .1491729864, .1527533871};
  cld cr;
  int i, i1, i2, j, j1, j2;
  cr = 0.;
  i1 = -20;
  for(i = 0; i < int(n); i++){
    i1 = i1 + 20;
    i2 = i1 + 19;
    for(j = 0; j < CONST2; j++){
      j1 = i1 + j;
      j2 = i2 - j;
      cr = cr + w[j] * (fd_c[j1] + fd_c[j2]);
    }
  }
  int_result = cr*.5l*(xhigh-xlow)/n;
}

//  Reads in the incoming photon 4-momentum and outgoing electron-positron 4-momenta.
//  Returns the J/psi scattering angle in the center-of-mass frame.
cld get_cosCM(cld * parameters)
{
  cld Eg  = parameters[0];
  cld q1g  = parameters[1];
  cld q2g  = parameters[2];
  cld q3g  = parameters[3];
  cld mpro  = 0.9383l;
  cld Epsi  = parameters[4]+parameters[8];
  cld p3x  = parameters[5]+parameters[9];
  cld p3y  = parameters[6]+parameters[10];
  cld p3z  = parameters[7]+parameters[11];
  cld mpsi  = 3.097l;
  
  cld result = (1.l - (2.l*Eg*mpro*(2.l*Eg + mpro)*(Sqrt(Power(Epsi,2) - Power(mpsi,2)) - p3z))/
    (Sqrt(Power(Eg,2)*Power(mpro,2))*
      Sqrt(4.l*Power(Eg,2)*Power(mpro,2) - 4.l*Eg*mpro*Power(mpsi,2) - 
        4.l*Power(mpro,2)*Power(mpsi,2) + Power(mpsi,4)))
	);
  return result;
}

//  Reads in the energy and cosine, as well as the input parameters, and delta- and d-functions.
//  Returns the amplitude of photoproduction.
cld AmpFunc(cld egam, cld cos, cld * input, cld deltafunc, cld dfunc)
{
  cld a0 = input[0];
  cld ap = input[1];
  cld anorm = input[2];
  cld st = input[3];
  cld b0 = input[4];
  cld xBrn = input[5];
  cld xMassr = input[6];
  cld xGamr = input[7];
  cld jr = input[8];
  cld angmom;
  if (jr == 1.5l)
   {
      angmom = 0.l;
   }
  else
   {
      angmom = 1.l;
   }
  
  cld result = Complex(0,1)*anorm*deltafunc*Power(E,(b0*(-1.l + cos)*egam*xMpro*
        Sqrt((Power(xMjpsi,4) + 4.l*Power(egam,2)*Power(xMpro,2) - 
            4.l*Power(xMjpsi,2)*xMpro*(egam + xMpro))/(xMpro*(2.l*egam + xMpro))))/
      Sqrt(xMpro*(2.l*egam + xMpro)))*Power(-st + xMpro*(2.l*egam + xMpro),
     a0 + (ap*(egam*Power(xMjpsi,2) - 2.l*Power(egam,2)*xMpro + Power(xMjpsi,2)*xMpro + 
           cos*egam*Sqrt(xMpro*(2.l*egam + xMpro))*
            Sqrt((Power(xMjpsi,4) + 4.l*Power(egam,2)*Power(xMpro,2) - 
                4.l*Power(xMjpsi,2)*xMpro*(egam + xMpro))/(xMpro*(2.l*egam + xMpro)))))/(2.l*egam + xMpro))
     + (2.l*alpha*Sqrt(2.l)*dfunc*egam*Pi*Sqrt((1.l + 2.l*jr)*xBrn*xGamr)*Power(xMpro,2)*
      (-Power(xMassr,2) + Power(xMpro,2))*
      Sqrt((Power(4,1.l - angmom)*alpha*(1.l + 2.l*jr)*xBrn*Power(xFjpsi,2)*xGamr*Power(xMassr,3)*
          Power((Power(xMassr,2) - Power(xMpro,2))/
            (xMassr*Sqrt(-Power(xMjpsi,2) + 
                Power(Power(xMassr,2) + Power(xMjpsi,2) - Power(xMpro,2),2)/(4.l*Power(xMassr,2)))),
           1.l + 2.l*angmom))/(Power(xMjpsi,2)*xMpro*Power(Power(xMassr,2) - Power(xMpro,2),2))))/
    (3.l*Sqrt(xMassr*xMpro)*Power((alpha*egam*xMpro)/Sqrt(xMpro*(2.l*egam + xMpro)),1.5)*
      Power((Power(xMjpsi,4) + 4.l*Power(egam,2)*Power(xMpro,2) - 
          4.l*Power(xMjpsi,2)*xMpro*(egam + xMpro))/(xMpro*(2.l*egam + xMpro)),0.25)*
      (Complex(0,1)*xGamr*xMassr - Power(xMassr,2) + xMpro*(2.l*egam + xMpro)))
	       ;
  return result;
}

//Squares the amplitude
cld AmpFuncSqr(cld egam, cld cos, cld * input, cld deltafunc, cld dfunc)
{  
  cld result = AmpFunc(egam,cos,input,deltafunc,dfunc)*conj(AmpFunc(egam,cos,input,deltafunc,dfunc));
  return result;
}

//  Reads in the energy, cosine and input parameters.
//  Returns the DXS of photoproduction.
cld DXSFunc(cld egam, cld cos, cld * input)
{
  cld d3232p32p(cld cos);
  cld d3232p12p(cld cos);
  cld d3232p12m(cld cos);
  cld d3232p32m(cld cos);
  cld d3212p32p(cld cos);
  cld d3212p12p(cld cos);
  cld d3212p12m(cld cos);
  cld d3212p32m(cld cos);
  cld d3212m32p(cld cos);
  cld d3212m12p(cld cos);
  cld d3212m12m(cld cos);
  cld d3212m32m(cld cos);
  cld d3232m32p(cld cos);
  cld d3232m12p(cld cos);
  cld d3232m12m(cld cos);
  cld d3232m32m(cld cos);
  cld d5232p32p(cld cos);
  cld d5232p12p(cld cos);
  cld d5232p12m(cld cos);
  cld d5232p32m(cld cos);
  cld d5212p32p(cld cos);
  cld d5212p12p(cld cos);
  cld d5212p12m(cld cos);
  cld d5212p32m(cld cos);
  cld d5212m32p(cld cos);
  cld d5212m12p(cld cos);
  cld d5212m12m(cld cos);
  cld d5212m32m(cld cos);
  cld d5232m32p(cld cos);
  cld d5232m12p(cld cos);
  cld d5232m12m(cld cos);
  cld d5232m32m(cld cos);
  int i;
  cld norm = (6084.375l*alpha)/(pow(egam,2)*pow(xMpro,2));
//
  cld jr  = input[8];
  cld a12 = input[10];
  cld a32 = sqrt(1.l-pow(input[10],2));
  cld angmom;
  cld listdfunc[24];
  cld listdelfunc[24];

  listdelfunc[0]  = 1.l;
  listdelfunc[1]  = 0.l;
  listdelfunc[2]  = 0.l;
  listdelfunc[3]  = 0.l;
  listdelfunc[4]  = 0.l;
  listdelfunc[5]  = 0.l;
  listdelfunc[6]  = 0.l;
  listdelfunc[7]  = 1.l;
  listdelfunc[8]  = 0.l;
  listdelfunc[9] = 0.l;
  listdelfunc[10] = 0.l;
  listdelfunc[11] = 0.l;
  listdelfunc[12] = 0.l;
  listdelfunc[13] = 0.l;
  listdelfunc[14] = 0.l;
  listdelfunc[15] = 1.l;
  listdelfunc[16] = 0.l;
  listdelfunc[17] = 0.l;
  listdelfunc[18] = 0.l;
  listdelfunc[19] = 0.l;
  listdelfunc[20] = 0.l;
  listdelfunc[21] = 0.l;
  listdelfunc[22] = 0.l;
  listdelfunc[23] = 1.l;
  if (jr == 1.5l)
   {
      angmom = 0.l;
  listdfunc[0] = d3232p32p(cos);
  listdfunc[1] = d3232p12p(cos);
  listdfunc[2] = d3232p12p(cos);
  listdfunc[3] = d3232p12m(cos);
  listdfunc[4] = d3232p12m(cos);
  listdfunc[5] = d3232p32m(cos);
  listdfunc[6] = d3212p32p(cos);
  listdfunc[7] = d3212p12p(cos);
  listdfunc[8] = d3212p12p(cos);
  listdfunc[9] = d3212p12m(cos);
  listdfunc[10] = d3212p12m(cos);
  listdfunc[11] = d3212p32m(cos);
  listdfunc[12] = d3212m32p(cos);
  listdfunc[13] = d3212m12p(cos);
  listdfunc[14] = d3212m12p(cos);
  listdfunc[15] = d3212m12m(cos);
  listdfunc[16] = d3212m12m(cos);
  listdfunc[17] = d3212m32m(cos);
  listdfunc[18] = d3232m32p(cos);
  listdfunc[19] = d3232m12p(cos);
  listdfunc[20] = d3232m12p(cos);
  listdfunc[21] = d3232m12m(cos);
  listdfunc[22] = d3232m12m(cos);
  listdfunc[23] = d3232m32m(cos);
   }
  else
   {
      angmom = 1.l;
  listdfunc[0] = d5232p32p(cos);
  listdfunc[1] = d5232p12p(cos);
  listdfunc[2] = d5232p12p(cos);
  listdfunc[3] = d5232p12m(cos);
  listdfunc[4] = d5232p12m(cos);
  listdfunc[5] = d5232p32m(cos);
  listdfunc[6] = d5212p32p(cos);
  listdfunc[7] = d5212p12p(cos);
  listdfunc[8] = d5212p12p(cos);
  listdfunc[9] = d5212p12m(cos);
  listdfunc[10] = d5212p12m(cos);
  listdfunc[11] = d5212p32m(cos);
  listdfunc[12] = d5212m32p(cos);
  listdfunc[13] = d5212m12p(cos);
  listdfunc[14] = d5212m12p(cos);
  listdfunc[15] = d5212m12m(cos);
  listdfunc[16] = d5212m12m(cos);
  listdfunc[17] = d5212m32m(cos);
  listdfunc[18] = d5232m32p(cos);
  listdfunc[19] = d5232m12p(cos);
  listdfunc[20] = d5232m12p(cos);
  listdfunc[21] = d5232m12m(cos);
  listdfunc[22] = d5232m12m(cos);
  listdfunc[23] = d5232m32m(cos);
   }
  cld sum_over_amp = 0.l;
  for(i=0;i<6;i++)
  {
	sum_over_amp = sum_over_amp + AmpFuncSqr(egam, cos, input, listdelfunc[i], listdfunc[i]*a32);
  }
  for(i=6;i<18;i++)
  {
	sum_over_amp = sum_over_amp + AmpFuncSqr(egam, cos, input, listdelfunc[i], listdfunc[i]*a12);
  }
  for(i=18;i<24;i++)
  {
	sum_over_amp = sum_over_amp + AmpFuncSqr(egam, cos, input, listdelfunc[i], listdfunc[i]*a32);
  }
  cld result = norm*sum_over_amp;
  return result;
}

//Reads energy, cosine and input parameters
//Returns the differential cross section with smearing
cld DXSSmearFunc(cld egam, cld cos, cld * input)
{
  cld smear = input[9];
  int i;
  ld enerd[CONST1], np;
  ld enerlow  =  real((Power(xMjpsi,2)+2.l*xMpro*xMjpsi)/(2.l*xMpro)); // lower bound
  ld enerhigh =  20.l; // higher bound
  ld n     =  ACCSMEAR; // accuracy
  DSG20R(enerlow, enerhigh, n, enerd, np); 

  cld den_values[CONST1];

  for(i = 0; i<int(np); i++){
    den_values[i] = DXSFunc(enerd[i], cos, input)*Exp(-Power(egam-enerd[i],2)/(2.l*Power(smear,2)))/(Sqrt(2.l*Pi)*smear);
  }

  cld result;
  DRG20_complex(enerlow, enerhigh, n, den_values, result); 

  return result;
}

//Reads energy and input parameters
//Returns the total cross section
cld TotXSFunc(cld egam, cld * input)
{
  int i;
  cld dtdcos =  (egam*(Sqrt(xMpro*(2.l*egam + xMpro))*
             Sqrt((Power(xMjpsi,4) + 4.l*Power(egam,2)*Power(xMpro,2) - 
                 4.l*Power(xMjpsi,2)*xMpro*(egam + xMpro))/
     	(xMpro*(2.l*egam + xMpro)))))/
       (2.l*egam + xMpro);
  ld cosd[CONST1], np;
  ld coslow  =  -1.l; // lower bound
  ld coshigh =  1.l; // higher bound
  ld n     =  ACCTOT; // accuracy
  DSG20R(coslow, coshigh, n, cosd, np); 

  cld den_values[CONST1];

  for(i = 0; i<int(np); i++){
    den_values[i] = DXSFunc(egam, cosd[i], input)*dtdcos;
  }

  cld result;
  DRG20_complex(coslow, coshigh, n, den_values, result); 

  return result;
}

//Reads energy and input parameters
//Returns the total cross section with smearing
cld TotXSSmearFunc(cld egam, cld * input)
{ 
  cld smear = input[9];
  int i;
  cld dtdcos =  (egam*(Sqrt(xMpro*(2.l*egam + xMpro))*
             Sqrt((Power(xMjpsi,4) + 4.l*Power(egam,2)*Power(xMpro,2) - 
                 4.l*Power(xMjpsi,2)*xMpro*(egam + xMpro))/
     	(xMpro*(2.l*egam + xMpro)))))/
       (2.l*egam + xMpro);
  ld cosd[CONST1], np;
  ld coslow  =  -1.l; // lower bound
  ld coshigh =  1.l; // higher bound
  ld n     =  ACCTOT; // accuracy
  DSG20R(coslow, coshigh, n, cosd, np); 

  cld den_values[CONST1];

  for(i = 0; i<int(np); i++){
    den_values[i] = DXSSmearFunc(egam, cosd[i], input)*dtdcos;
  }

  cld result;
  DRG20_complex(coslow, coshigh, n, den_values, result); 

  return result;
}

//d-functions
cld d3232p32p(cld cos)
{
  cld result = Power(1.l + cos,1.5)/(2.l*Sqrt(2.l));
  return result;
}

cld d3232p12p(cld cos)
{
  cld result = -(Sqrt(1.5l)*Sqrt(1.l - cos)*(1.l + cos))/2.l;   
  return result;
}

cld d3232p12m(cld cos)
{
  cld result = -(Sqrt(1.5l)*(-1.l + cos)*Sqrt(1.l + cos))/2.l;   
  return result;
}

cld d3232p32m(cld cos)
{
  cld result = -Power(1.l - cos,1.5)/(2.l*Sqrt(2.l));   
  return result;
}

cld d3212p32p(cld cos)
{
  cld result = (Sqrt(1.5l)*Sqrt(1.l - cos)*(1.l + cos))/2.l;	       
  return result;
}

cld d3212p12p(cld cos)
{
  cld result = (Sqrt(1.l + cos)*(-1.l + 3.l*cos))/(2.l*Sqrt(2.l));   
  return result;
}

cld d3212p12m(cld cos)
{
  cld result = -(Sqrt(2.l - 2.l*cos)*(1.l + 3.l*cos))/4.l;   
  return result;
}

cld d3212p32m(cld cos)
{
  cld result = -(Sqrt(1.5l)*(-1.l + cos)*Sqrt(1.l + cos))/2.l;   
  return result;
}

cld d3212m32p(cld cos)
{
  cld result = -(Sqrt(1.5l)*(-1.l + cos)*Sqrt(1.l + cos))/2.l;	       
  return result;
}

cld d3212m12p(cld cos)
{
  cld result = (Sqrt(2.l - 2.l*cos)*(1.l + 3.l*cos))/4.l;   
  return result;
}

cld d3212m12m(cld cos)
{
  cld result = (Sqrt(1.l + cos)*(-1.l + 3.l*cos))/(2.l*Sqrt(2.l));   
  return result;
}

cld d3212m32m(cld cos)
{
  cld result = -(Sqrt(1.5l)*Sqrt(1.l - cos)*(1.l + cos))/2.l;   
  return result;
}

cld d3232m32p(cld cos)
{
  cld result = Power(1.l - cos,1.5)/(2.l*Sqrt(2.l));	       
  return result;
}

cld d3232m12p(cld cos)
{
  cld result = -(Sqrt(1.5l)*(-1.l + cos)*Sqrt(1.l + cos))/2.l;   
  return result;
}

cld d3232m12m(cld cos)
{
  cld result = (Sqrt(1.5l)*Sqrt(1.l - cos)*(1.l + cos))/2.l;   
  return result;
}

cld d3232m32m(cld cos)
{
  cld result = Power(1.l + cos,1.5)/(2.l*Sqrt(2.l));   
  return result;
}

cld d5232p32p(cld cos)
{
  cld result = (Power(1.l + cos,1.5)*(-3.l + 5.l*cos))/(4.l*Sqrt(2.l));	       
  return result;
}

cld d5232p12p(cld cos)
{
  cld result = -(Sqrt(1.l - cos)*(1.l + cos)*(-1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5232p12m(cld cos)
{
  cld result = -((-1.l + cos)*Sqrt(1.l + cos)*(1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5232p32m(cld cos)
{
  cld result = -(Power(1.l - cos,1.5)*(3.l + 5.l*cos))/(4.l*Sqrt(2.l));   
  return result;
}

cld d5212p32p(cld cos)
{
  cld result = (Sqrt(1.l - cos)*(1.l + cos)*(-1.l + 5.l*cos))/4.l;	       
  return result;
}

cld d5212p12p(cld cos)
{
  cld result = (Sqrt(1.l + cos)*(-1.l - 2.l*cos + 5.l*Power(cos,2)))/(2.l*Sqrt(2.l));   
  return result;
}

cld d5212p12m(cld cos)
{
  cld result = (Sqrt(2.l - 2.l*cos)*(1.l - 2.l*cos - 5.l*Power(cos,2)))/4.l;   
  return result;
}

cld d5212p32m(cld cos)
{
  cld result = -((-1.l + cos)*Sqrt(1.l + cos)*(1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5212m32p(cld cos)
{
  cld result = -((-1.l + cos)*Sqrt(1.l + cos)*(1.l + 5.l*cos))/4.l;	       
  return result;
}

cld d5212m12p(cld cos)
{
  cld result = (Sqrt(2.l - 2.l*cos)*(-1.l + 2.l*cos + 5.l*Power(cos,2)))/4.l;   
  return result;
}

cld d5212m12m(cld cos)
{
  cld result = (Sqrt(1.l + cos)*(-1.l - 2.l*cos + 5.l*Power(cos,2)))/(2.l*Sqrt(2.l));   
  return result;
}

cld d5212m32m(cld cos)
{
  cld result = -(Sqrt(1.l - cos)*(1.l + cos)*(-1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5232m32p(cld cos)
{
  cld result = (Power(1.l - cos,1.5)*(3.l + 5.l*cos))/(4.l*Sqrt(2.l));	       
  return result;
}

cld d5232m12p(cld cos)
{
  cld result = -((-1.l + cos)*Sqrt(1.l + cos)*(1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5232m12m(cld cos)
{
  cld result = (Sqrt(1.l - cos)*(1.l + cos)*(-1.l + 5.l*cos))/4.l;   
  return result;
}

cld d5232m32m(cld cos)
{
  cld result = (Power(1.l + cos,1.5)*(-3.l + 5.l*cos))/(4.l*Sqrt(2.l));   
  return result;
}
